#!/usr/bin/env python
# -*- coding: utf-8 -*-

# this must come first
from __future__ import print_function, unicode_literals, division

# standard library imports
from argparse import ArgumentParser
import os.path
import shutil
import json


########################################
def parse_myproject(filename, project):

  logins  = {}

  with open(filename, "r") as filein:
    # Skip lines until we find project name.
    # Then extract script date
    for ligne in filein:
      if project["project"] in ligne:
        today = ligne.split()[-1]
        break

    # Skip next two lines : first is blank and second is login titles
    for _ in xrange(1):
      next(filein)

    # Login list
    for ligne in filein:
      if not ligne.strip():
        break
      login, conso = ligne.split()
      logins[login] = float(conso)

    # Skip all the rest until we find total
    for ligne in filein:
      if "Total" in ligne:
        total = float(ligne.split()[-1])
        break

    # Skip all the rest until we find deadline
    for ligne in filein:
      if "Allocated" in ligne:
        project["alloc"] = float(ligne.split()[-1])
        break

    # Skip all the rest until we find deadline
    for ligne in filein:
      if "Suggested use at this time" in ligne:
        utheo = float(ligne.split()[-1].strip("%"))
        break

    # Skip all the rest until we find deadline
    for ligne in filein:
      if "Real use at this time" in ligne:
        ureal = float(ligne.split()[-1].strip("%"))
        break

    # Skip all the rest until we find deadline
    for ligne in filein:
      if "Project deadline" in ligne:
        project["deadline"] = ligne.split()[-1]
        break

  return today, total, utheo, ureal, logins


########################################
def write_param(filename, project):

  if args.dryrun:
    print(json.dumps(project, indent=2))
  else:
    with open(filename, "w") as fileout:
      json.dump(project, fileout, indent=2)


########################################
def write_bilan(filename, today, total, ureal, utheo):
  """
  Conso totale par jour
  ---------------------
  on garde le total, date en tete en accumulant dans le fichier :
  OUT_CONSO_BILAN
  """

  title_str  = "{:10s} {:12s} {:11s} {:11s}\n".format(
                 "date",
                 "conso(hours)",
                 "real_use(%)",
                 "theo_use(%)",
               )
  result_str = "{:10s} {:12.2f} {:11.2f} {:11.2f}\n".format(
                 today,
                 total,
                 ureal,
                 utheo,
               )

  if args.dryrun:
    print(title_str.strip())
    print(result_str.strip())
  else:
    if not os.path.isfile(filename):
      with open(filename, "w") as fileout:
        fileout.write(title_str)
    with open(filename, "a") as fileout:
      fileout.write(result_str)


########################################
def write_utheo(filename, today, utheo):
  """
  Conso théorique par jour
  ------------------------
  OUT_CONSO_THEO
  """

  title_str  = "{:10s} {:11s}\n".format(
                 "date",
                 "theo_use(%)",
               )
  result_str = "{:10s} {:11.2f}\n".format(
                 today,
                 utheo,
               )

  if args.dryrun:
    print(title_str.strip())
    print(result_str.strip())
  else:
    if not os.path.isfile(filename):
      with open(filename, "w") as fileout:
        fileout.write(title_str)
    with open(filename, "a") as fileout:
      fileout.write(result_str)


########################################
def write_login(filename, today, logins):
  """
  Conso par login (HOME)
  ----------------------
  on garde la trace de chaque login, date en tete, en remplacant
  le fichier a chaque fois : OUT_CONSO_LOGIN
  """

  title_str  = "{:10s} {:10s} {:12s}\n".format(
                 "date",
                 "login",
                 "conso(hours)",
               )

  with open(filename, "w") as fileout:
    if args.dryrun:
      print(title_str.strip())
    else:
      fileout.write(title_str)

    for key in sorted(logins):
      result_str = "{:10s} {:10s} {:12.2f}\n".format(
                     today,
                     key,
                     logins[key],
                   )
      if args.dryrun:
        print(result_str.strip())
      else:
        fileout.write(result_str)


########################################
def write_store(filename):
  """
  volume cree sur STORE
  ---------------------
  par login qui a consomme, en remplacant le fichier a chaque fois :
  OUT_CONSO_STORE
  """

  pass


########################################
if __name__ == '__main__':

  # Get arguments from command line
  # ===============================
  parser = ArgumentParser()
  parser.add_argument("-v", "--verbose", action="store_true",
                      help="Verbose mode")
  parser.add_argument("-d", "--dryrun", action="store_true",
                      help="dry run, no file produced")
  parser.add_argument("-a", "--all", action="store_false",
                      help="produce all files (default)")
  parser.add_argument("-b", "--bilan", action="store_true",
                      help="produce all files (default)")
  parser.add_argument("-l", "--login", action="store_true",
                      help="produce all files (default)")
  parser.add_argument("-s", "--store", action="store_true",
                      help="produce all files (default)")

  args = parser.parse_args()
  if args.verbose:
    print(os.path.basename(__file__))
    print(args)

  # Files and directories
  # =====================
  LOCAL_DIR = os.path.join(
    "/ccc", "cont003", "home", "dsm", "p86ipsl", "ConsoGENCMIP6", "output"
    # "/home_local", "slipsl", "ConsoGENCMIP6", "output"
  )
  SAVE_DIR  = os.path.join(
    "/ccc", "work", "cont003", "dsm", "p86ipsl", "ConsoGENCMIP6",
    # "/home_local", "slipsl", "ConsoGENCMIP6", "save"
  )

  OUT_PARAM = "OUT_CONSO_PARAM"
  OUT_BILAN = "OUT_CONSO_BILAN"
  OUT_UTHEO = "OUT_CONSO_UTHEO"
  OUT_LOGIN = "OUT_CONSO_LOGIN"
  OUT_STORE = "OUT_CONSO_STORE"

  if args.verbose:
    print(LOCAL_DIR)
    print(SAVE_DIR)

  ccc_myproject = "ccc_myproject.dat"

  project = {}
  project["project"] = "gencmip6"

  (today, total, utheo, ureal, logins) = \
      parse_myproject(
        os.path.join(LOCAL_DIR, ccc_myproject),
        project
      )

  if args.verbose:
    print(today, utheo, ureal)
    print(project)
    print(logins)

  # Produce files
  # =============

  # 1- Parametres du projet
  # -----------------------
  write_param(os.path.join(LOCAL_DIR, OUT_PARAM), project)

  # 2- Conso totale par jour
  # ------------------------
  write_bilan(
    os.path.join(LOCAL_DIR, OUT_BILAN),
    today,
    total,
    ureal,
    utheo
  )

  # 2b- Conso théorique par jour
  # ----------------------------
  write_utheo(os.path.join(LOCAL_DIR, OUT_UTHEO), today, utheo)

  # 3- Conso par login (HOME)
  # -------------------------
  write_login(os.path.join(LOCAL_DIR, OUT_LOGIN), today, logins)

  # 4- volume cree sur STORE
  # ------------------------
  write_store(os.path.join(LOCAL_DIR, OUT_STORE))

  # Save files (on WORKDIR)
  # =======================
  if not args.dryrun:
    suffix = today.replace("-", "")
    file_list = [
      OUT_PARAM,
      OUT_BILAN,
      OUT_UTHEO,
      OUT_LOGIN,
      OUT_STORE
    ]
    for filename in file_list:
      filein  = os.path.join(LOCAL_DIR, filename)
      if os.path.isfile(filein):
        fileout = os.path.join(SAVE_DIR, "_".join((filename, suffix)))
        shutil.copy(filein, fileout)
